#!/bin/bash

# Define repository URLs and target directories
repository_url_external="ssh://git@gitlab.sw-motion.cn:24/jac-soc/external-dependencies.git"
repository_url_calibrations="ssh://git@gitlab.sw-motion.cn:24/jac-soc/calibrations.git"
target_dir="/home/mdc/ext_dependencies"
hosts=("192.168.13.1")
local_external_deps_dir=~/mdc_external_deps
original_dir=$(pwd)
# Function to fetch the remote branch commit hash
fetch_remote_commit() {
    git ls-remote $1 $2 | awk '{print $1}'
}

# Function to get the repository branch name
get_repo_branch() {
    git rev-parse --abbrev-ref HEAD
}

# Function to get the latest commit hash
get_repo_commit() {
    git rev-parse --short HEAD
}

# Function to update repository
update_repository() {
    local repo_url=$1
    local branch_name=$2
    local repo_folder=$(basename $repo_url .git)
    

    if [ -d "$repo_folder" ]; then
        echo "Repository directory '$repo_folder' already exists."
        cd "$repo_folder" || { echo "Failed to navigate to '$repo_folder'"; exit 1; }
        echo "Fetching latest changes from origin..."
        git fetch

        local current_branch=$(get_repo_branch)

        if [ -n "$branch_name" ]; then
            if [ "$current_branch" != "$branch_name" ]; then
                echo "Switching to branch '$branch_name'..."
                git checkout $branch_name || { echo "Failed to switch to branch '$branch_name'"; exit 1; }
            fi
            remote_commit=$(fetch_remote_commit $repo_url $branch_name)
            local_commit=$(get_repo_commit)
            if [ "$local_commit" != "$remote_commit" ]; then
                echo "Pulling latest updates for branch '$branch_name'..."
                git pull origin $branch_name
            else
                echo "Already up to date."
            fi
        else
            echo "Current branch is '$current_branch'."
            echo "No branch name provided. Do you want to:"
            echo "1. Stay on the current branch ('$current_branch')"
            echo "2. Switch to a different branch"

            read -p "Please enter your choice (1 or 2): " choice

            case $choice in
                1)
                    remote_commit=$(fetch_remote_commit $repo_url)
                    local_commit=$(get_repo_commit)
                    if [ "$local_commit" != "$remote_commit" ]; then
                        echo "Pulling latest updates for branch '$current_branch'..."
                        git pull origin $current_branch
                    else
                        echo "Already up to date."
                    fi
                    ;;
                2)
                    echo "Fetching branch list..."
                    git fetch --all
                    git branch -r

                    read -p "Enter the branch name you want to switch to: " selected_branch
                    if [ "$current_branch" != "$selected_branch" ]; then
                        echo "Switching to branch '$selected_branch'..."
                        git checkout "$selected_branch" || { echo "Failed to switch to branch '$selected_branch'"; exit 1; }
                    fi

                    remote_commit=$(fetch_remote_commit $repo_url $selected_branch)
                    local_commit=$(get_repo_commit)
                    if [ "$local_commit" != "$remote_commit" ]; then
                        echo "Pulling latest updates for branch '$selected_branch'..."
                        git pull origin "$selected_branch" || { echo "Failed to pull updates for '$selected_branch'"; exit 1; }
                    else
                        echo "Already up to date."
                    fi
                    ;;
                *)
                    echo "Invalid choice. Exiting."
                    exit 1
                    ;;
            esac
        fi
    else
        echo "Repository directory '$repo_folder' does not exist. Cloning repository..."
        git clone $repo_url

        if [ $? -eq 0 ]; then
            echo "Repository cloned successfully into $repo_folder"
            cd $repo_folder
            git checkout $branch_name || { echo "Failed to switch to branch '$branch_name'"; exit 1; }
        else
            echo "Failed to clone repository. Please check the branch name and try again."
            exit 1
        fi
    fi
}

# Function to copy files to a remote server
copy_to_server() {
    local repo_name=$1
    local target_dir=$2
    local host=$3
    
    if sshpass -p my_password ssh root@$host "[ -d \"$target_dir\" ]"; then
        echo "Removing existing files on $host...  $target_dir/*"
        sshpass -p my_password ssh root@$host "rm -rf $target_dir/*"

    else
        echo "Creating $target_dir"
        sshpass -p my_password ssh root@$host "mkdir -p $target_dir"
    fi
    echo "Copying new files to $host..."
    sshpass -p my_password scp -r $local_external_deps_dir/* root@$host:$target_dir

    echo "Removing extra files from the boards"
    sshpass -p my_password ssh root@$host "rm -rf $target_dir/external-dependencies/calibrations/.git $target_dir/external-dependencies/.git $target_dir/test.sh"

    if [ $? -ne 0 ]; then
        echo "Error: Failed to copy folders to $host"
        exit 1
    fi
}

# Creates external dependencies folder if not exists
mkdir -p "$local_external_deps_dir"
if [ -d "$local_external_deps_dir" ]; then
    echo "$local_external_deps_dir created successfully"
else
    echo "Something went wrong creating directory"
    exit 0
fi
cd $local_external_deps_dir
echo "current path is: $(pwd)"

# Update external-dependencies repository
update_repository $repository_url_external $1

# Update calibrations repository
update_repository $repository_url_calibrations $1

# Copy external-dependencies to remote hosts
for host in "${hosts[@]}"; do
    copy_to_server "external-dependencies" "$target_dir" $host
done
echo "Successfully copied files to boards"